/*
 * index function for BFS
 *
 */
#include <asm/uaccess.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/bfs_fs.h>
#include <linux/sched.h>
#include <linux/stat.h>
#include <linux/mm.h>


/*
 * read index node from i-node.
 *
 * inode -> address of index node
 * sb    -> super block
 * flags -> 0 ... inode is index entry
 *          1 ... inode is index node
 */
struct buffer_head * bfs_read_index_node( bfs_inode_addr inode,
	struct super_block * sb, int flags )
{
	struct buffer_head * bh;
	bfs_inode_addr       iaddr;
	bfs_index_entry *    entry;

	BFS_OUTPUT (("---> Enter bfs_read_index_node  "));
	BFS_OUTPUT (("index %lu, %u\n", inode.allocation_group, inode.start));

	if (!flags) {
		/*
		 * read index entry
		 */
		bh = bfs_bread2 (sb, inode);
		entry = (bfs_index_entry *) bh->b_data;

		BFS_DUMP_INDEX_ENTRY (entry);

		/*
		 * check magic header
		 */
		if (entry->magic != BFS_INDEX_MAGIC) {
			/* bad magic header */
			printk ("BFS: index entry's magic header is bad\n");
			brelse(bh);

			return NULL;
		}

		iaddr = inode;
		iaddr.start++;
		iaddr.len--;
	
		brelse(bh);
	} else {
		BFS_OUTPUT (("  skip index node\n"));
		iaddr = inode;
	}

	BFS_OUTPUT (("inode of index node\n"));
	BFS_DUMP_INODE_ADDR (iaddr);

	bh = bfs_bread2 (sb, iaddr);

	return bh;
}



/*
 * Get key form index node
 *
 *    bfs_index_node * bn    in
 *    int            * pos   in
 *    char           * name  out
 *    char           * name  out
 *    bfs_inodeaddr  * iaddr out
 */

char * bfs_get_key_from_index_node (bfs_index_node * bn, int *pos,
	char * name, int * len, bfs_off_t * iaddr)
{
	char *      key;
	short *     key_array;
	bfs_off_t * key_value;
	char *      namep;
	int         i;

	BFS_OUTPUT (("---> Enter bfs_get_key_from_index_node "));
	BFS_OUTPUT (("pos %d\n", *pos));

	if (*pos < 0 || bn->all_key_count <= *pos) {
		*pos = -1;
		return NULL;
	}

	key = ((char *) bn) + sizeof(bfs_index_node);
	key_array = (short *) (((unsigned long) key) + bn->all_key_length
		+ (8 - ((bn->all_key_length + sizeof(bfs_index_node)) % 8)));
	key_value = (bfs_inode_addr *) (((char *) key_array)
		+ sizeof(short) * bn->all_key_count);

	BFS_OUTPUT (("key %08x key_array %08x key_value %08x\n",
		(long) key, (long) key_array, (long) key_value));

	if( *pos == 0 ) {
		namep = key;
		*len = key_array[*pos];
	} else {
		namep = key + key_array[*pos - 1];
		*len = key_array[*pos] - key_array[*pos - 1];
	}

	/*
	 * copy from namep to name
	 */
	for( i = 0; i < *len; i++) {
		name[i] = namep[i];
	}

	name[i] = '\0';
	*iaddr = key_value[*pos];
	(*pos)++;

	BFS_OUTPUT (("  result key %s, value %Lu\n", name, *iaddr));

	return name;
}
